package Post

import (
	"github.com/nspcc-dev/neo-go/pkg/interop"
	"github.com/nspcc-dev/neo-go/pkg/interop/iterator"
	"github.com/nspcc-dev/neo-go/pkg/interop/native/gas"
	"github.com/nspcc-dev/neo-go/pkg/interop/native/std"
	"github.com/nspcc-dev/neo-go/pkg/interop/runtime"
	"github.com/nspcc-dev/neo-go/pkg/interop/storage"
)

type Post struct {
	postName string
	text     string
	login    string
	likes    int
	id       string
}

const (
	lastIndex      = "user_post_index"
	id             = "current_post_id"
	comment_prefix = "comment_"
	gas_decimals   = 1_0000_0000
)

func NewPost(login string, text string, postName string) {
	ctx := storage.GetContext()
	updatePostId()
	id := storage.Get(ctx, id).(int)
	post_id := "post_" + std.Itoa10(id)

	post := Post{
		text:     text,
		login:    login,
		likes:    0,
		postName: postName,
		id:       post_id,
	}

	storage.Put(ctx, post.id, std.Serialize(post))

	updatePostIndex(login)
	lastPostIndex := storage.Get(ctx, login+lastIndex).(int)
	storage.Put(ctx, login+"_p_"+std.Itoa10(lastPostIndex), post.id)
	runtime.Log("Post created. Post id: " + post.id + "; User post id" + login + "_p_" + std.Itoa10(lastPostIndex))
}

func GetAllPosts() []Post {
	posts := make([]Post, 0)
	ctx := storage.GetReadOnlyContext()
	it := storage.Find(ctx, "post", storage.ValuesOnly|storage.DeserializeValues)
	for iterator.Next(it) {
		post := iterator.Value(it).(Post)
		runtime.Log(string(std.Serialize(post)))
		posts = append(posts, post)
	}

	return posts
}

func GetPost(postId string) Post {

	ctx := storage.GetReadOnlyContext()
	data := storage.Get(ctx, postId)
	if data == nil {
		panic("post not found")
	}

	return std.Deserialize(data.([]byte)).(Post)
}

func GetAllPostsByUser(login string) []Post {
	ctx := storage.GetReadOnlyContext()
	var postsByUser []Post = make([]Post, 0)
	i := 1
	n := getPostIndex(login)
	for i < n+1 {
		post_key := storage.Get(ctx, login+"_p_"+std.Itoa10(i))
		if post_key == nil {
			panic("post key not found")
		}

		post_key = post_key.(string)

		post := storage.Get(ctx, post_key)
		if post == nil {
			panic("post not found")
		}

		i++
		postsByUser = append(postsByUser, std.Deserialize(post.([]byte)).(Post))
	}
	return postsByUser
}

func Rate(postId string, walletHashFrom interop.Hash160) bool {
	ctx := storage.GetContext()
	post := GetPost(postId)
	recipient := storage.Get(ctx, post.login+"_hash").(interop.Hash160)
	success := gas.Transfer(walletHashFrom, recipient, 1, nil)
	if !success {
		return false
	} else {
		return true
	}
}

func IncreaseRate(postId string) {
	ctx := storage.GetContext()
	post := GetPost(postId)
	post.likes += 1
	storage.Put(ctx, post.id, std.Serialize(post))

}

func getPostIndex(userId string) int {
	ctx := storage.GetContext()
	index := storage.Get(ctx, userId+lastIndex)
	if index == nil {
		index = 0
	}
	return index.(int)
}

func updatePostIndex(userId string) {
	ctx := storage.GetContext()
	index := getPostIndex(userId)
	storage.Put(ctx, userId+lastIndex, index+1)
}

func getPostId() int {
	ctx := storage.GetContext()
	current_id := storage.Get(ctx, id)
	if current_id == nil {
		current_id = 0
	}
	return current_id.(int)
}

func updatePostId() {
	ctx := storage.GetContext()
	cur_id := getPostId()
	storage.Put(ctx, id, cur_id+1)
}
